# vim: set fileencoding=utf-8 :

import sys
from katrin.models import *
from katrin.service import ServiceStat
from itertools import groupby
from datetime import datetime

class GeneralScript(object):
	thescript = ''
	@classmethod
	def print_help(cls):
		childs = []
		for child in cls.__subclasses__():
			childs.append(child.__name__)
		print """== Syntax ==
	%s <script name>' <script options>
		script name
			Use one of the following names:
			%s
		""" % (sys.argv[0], childs)
	
	@classmethod
	def check_args(cls):
		if len(sys.argv) < 2:
			cls.print_help()
			exit(1)
		this_module = sys.modules[__name__]
		if  hasattr(this_module, sys.argv[1]):
			klass = getattr(this_module, sys.argv[1])
			if issubclass(klass,cls):
				cls.thescript = klass
				cls.thescript.check_args()
			else:
				exit(2)

	@classmethod
	def run(cls):
		cls.thescript.run()

	@classmethod
	def validate_period(cls,duration):
		periods = BillingPeriod.objects.all()
		for period in periods:
			if duration == period.duration:
				return period.duration

		print u"Unknown duration %s" % duration
		print "Available durations:"
		for period in periods:
			print period.duration
		cls.print_help()
		exit(2)

class AggregateStat(GeneralScript):
	services = []
	for s in Service.objects.all():
		services.append(s.codename.__str__()) 
	@classmethod
	def print_help(cls):
		print """== Syntax ==
	%s %s <service> <aggregation level> '<entry time>'
		service
			Possible values: %s
		aggregation level
			Use one of the durations: %s
		entry time
			time for aggregation entry, format is "Y-m-d H:M:S" """ % (sys.argv[0], cls.__name__, cls.services, Period.Divisibles.keys())

	@classmethod
	def check_args(cls):
		if len(sys.argv) < 5:
			cls.print_help()
			exit(1)
		if sys.argv[2] not in cls.services or sys.argv[3] not in Period.Divisibles.keys():
			cls.print_help()
			exit(2)

	@classmethod
	def run(cls):
		service = sys.argv[2]
		level = sys.argv[3]
		try:
			dt = datetime.strptime(sys.argv[4], "%Y-%m-%d %H:%M:%S")
			ServiceStat.aggregate_stat(service,level,dt)
		except ValueError:
			print "Check your entry time."
			cls.print_help()

class CleanStat(GeneralScript):
	services = []
	start_time = "1970-01-01 00:00:00"
	end_time = ''
	for s in Service.objects.all():
		services.append(s.codename.__str__()) 
	@classmethod
	def print_help(cls):
		print """== Syntax ==
	%s %s <service> '<end time>' ['<start time>']
		service
			Possible values: %s
		end time
			end time for cleaning, format is "Y-m-d H:M:S
		start time
			optional start time for cleaning, format is "Y-m-d H:M:S """ % (sys.argv[0], cls.__name__, cls.services)

	@classmethod
	def check_args(cls):
		if len(sys.argv) < 4:
			cls.print_help()
			exit(1)
		cls.end_time = sys.argv[3]
		if len(sys.argv) == 5:
			cls.start_time = sys.argv[4]
		if sys.argv[2] not in cls.services:
			cls.print_help()
			exit(2)

	@classmethod
	def run(cls):
		service = sys.argv[2]
		try:
			cls.end_time = datetime.strptime(cls.end_time, "%Y-%m-%d %H:%M:%S")
		except ValueError:
			print "Check your end time."
			cls.print_help()
		try:
			cls.start_time = datetime.strptime(cls.start_time, "%Y-%m-%d %H:%M:%S")
		except ValueError:
			print "Check your start time."
			cls.print_help()

		ServiceStat.clean_stat(service,cls.start_time,cls.end_time)

class CloseBillingPeriod(GeneralScript):
	@classmethod
	def print_help(cls):
		print """== Syntax ==
	%s %s '<billing period duration>'
		billing period duration
			Use one of the durations stored in the database.
		""" % (sys.argv[0], cls.__name__)

	@classmethod
	def check_args(cls):
		if len(sys.argv) < 3:
			cls.print_help()
			exit(1)

	@classmethod
	def run(cls):
		duration = unicode(sys.argv[2],'utf-8')
		duration = cls.validate_period(duration)
		userservices = UserService.objects.filter(tariff__billing_period__duration__exact=duration).order_by('account')
		if userservices.count() == 0:
			pass
			#print u"There is no userservice with billing period %s" % duration
		else:
			uservices = filter(lambda us: us.is_valid(), userservices)
			for ac, i in groupby(uservices, lambda us: us.account):
				ac.close_billing_period(list(i))

class ExportBills(GeneralScript):
	@classmethod
	def print_help(cls):
		print """== Syntax ==
	%s %s <user script>
		user script
			First try to specify "demo"
			Create your own script to export bills to your system (see demo script)
		Call this script AFTER billing period closure
		""" % (sys.argv[0], cls.__name__)

	@classmethod
	def check_args(cls):
		if len(sys.argv) < 3:
			cls.print_help()
			exit(1)

	@classmethod
	def run(cls):
		module_name = 'userbills.%s' % sys.argv[2]

		try:
			module = __import__(module_name, globals(), locals(), ["ExporterClass"])
		except(ImportError):
			print "Failed to load module %s" % module_name
			exit(3)

		exporter_class = getattr(module, "ExporterClass")
		closure = Operation.get_last_closure()
		if closure:
			bills = Bill.objects.filter(date__gte=closure.date_time.date())
			for bill in bills:
				exporter_class.export(bill)
		else:
			print "Call this script AFTER billing period closure"
